import random
import os
from rdkit import Chem
from Posit.utils.RMSD import calcRMSD


def download_data(number):
    pass


def prepare_data(play_dir, pdb_dir):
    ligands = os.path.join(play_dir, pdb_dir, 'ligands.sdf')
    mols = [i for i in Chem.SDMolSupplier(ligands)]
    mols_h = [i for i in Chem.SDMolSupplier(ligands, removeHs=False)]
    ref = mols[:][0]
    atoms = [i for i in range(ref.GetNumAtoms())]
    random.shuffle(mols)

    # calc rmsd
    best_mol = None
    rmsd_value = {}
    w = Chem.SDWriter(ligands)
    for idx, mol in enumerate(mols):
        name = str(idx + 1)
        # rename mols
        mols_h[idx].SetProp('_Name', name)
        atomMaps = list(zip(atoms, atoms))
        rmsd = calcRMSD(mol, ref, atomMaps)
        rmsd_value[name] = rmsd
        # print(idx + 1, rmsd)
        if rmsd == 0:
            best_mol = name
        w.write(mols_h[idx])
    w.close()
    # rmsd_value - {"name": rmsd}
    return [rmsd_value, best_mol]


import os
import shutil


def dry(datapath, tmp_dir, savedpath):
    error = 0
    for i in os.listdir(datapath):
        pdb = os.path.join(datapath, i)
        tmp_pdb = os.path.join(tmp_dir, 'dry_' + i)
        try:
            os.system(f'pdb4amber -i {pdb} -o {tmp_pdb} -d')
            shutil.copy(tmp_pdb, savedpath)
        except Exception as e:
            print(i, repr(e))
            error += 1

    print(error)


if __name__ == '__main__':
    datapath = 'kinase_pdbs'
    savedpath = 'kinase_pdbs_dry'
    tmp_dir = 'tmp'

    dry(datapath, tmp_dir, savedpath)






import math


def dis_p2p(p1, p2):
    r = math.pow((p1[0] - p2[0]), 2) + math.pow((p1[1] - p2[1]), 2) + math.pow((p1[2] - p2[2]), 2)
    return r


def calcRMSD(proMol, refMol, atomMaps):
    ref_conf = refMol.GetConformer()
    mol_conf = proMol.GetConformer()
    mse = 0
    for i, j in atomMaps:
        pos1 = mol_conf.GetAtomPosition(i)
        pos2 = ref_conf.GetAtomPosition(j)
        p1 = [pos1.x, pos1.y, pos1.z]
        p2 = [pos2.x, pos2.y, pos2.z]
        r = dis_p2p(p1, p2)
        mse += r
    mse /= len(atomMaps)
    RMSD = math.sqrt(mse)
    return round(RMSD, 3)


import click
import os
import datetime
from Posit.utils.prepare import prepare_data, download_data


def print_answer(user_input, best_mol, rmsd, score, idx):
    print_list = []
    if user_input != best_mol:
        print_list.append('\nMol %s RMSD: %s' % (user_input, rmsd[user_input]))
        print_list.append('Best Mol: %s' % best_mol)
        print_list.append('Your score of Exercise_%s: %s' % ((idx + 1), '待开发'))
    else:
        print_list.append('\nCorrect answer, Prefect!')
    print_list.append('\nExercise_%s Answer List:' % (idx + 1))
    for i in rmsd.keys():
        print_list.append(i + ':' + str(rmsd[i]))

    for i in print_list:
        print(i)
    return print_list


@click.command()
@click.option('-n', '--number', type=int, help='Number of Exercise', default=5)
def run(number):
    # download data from CosMos
    download_data(number)

    answer = []
    play_dir = 'Posit'
    pdb_dir_list = [i for i in os.listdir(play_dir) if os.path.isdir(os.path.join(play_dir, i))]

    for idx, pdb_dir in enumerate(pdb_dir_list):
        old_dir_name = os.path.join(play_dir, pdb_dir)
        new_dir_name = os.path.join(play_dir, 'Exercise_'+str(idx+1))
        answer.append(prepare_data(play_dir, pdb_dir))
        os.rename(old_dir_name, new_dir_name)

    txt_list = []

    # get start time
    start = datetime.datetime.now()

    answer_area = [str(i) for i in range(1, 11)]
    for idx, ans in enumerate(answer):
        rmsd = ans[0]
        best_mol = ans[1]
        while True:
            user_input = input('\nInput the mol number of Exercise_%s:' % (idx+1))
            if user_input in answer_area:
                break
            else:
                print('Input error!Please input the number!')

        # record
        txt_list.append('-------------')
        txt_list.append('Exercise_%s' % (idx + 1))
        txt_list.append("User's choice: %s" % user_input)

        # calc score
        score = None

        # print_answer
        print_list = print_answer(user_input, best_mol, rmsd, score, idx)
        txt_list.extend(print_list)
        txt_list.append('-------------')

    print('Your total score: %s' % '待开发')
    txt_list.append('\n\nTotal score: %s\n' % '待开发')

    # get end time and time cost
    end = datetime.datetime.now()
    txt_list.append('start time:' + start.strftime('%Y.%m.%d-%H:%M:%S'))
    txt_list.append('end time:' + end.strftime('%Y.%m.%d-%H:%M:%S'))
    time_cost = 'time cost: ' + str(end - start)
    txt_list.append(time_cost)
    print(time_cost)

    # save to a txt file
    txt_list = [i+'\n' for i in txt_list]
    with open(os.path.join(play_dir, 'Posit_record.txt'), 'w') as f:
        f.writelines(txt_list)


if __name__ == '__main__':
    run()



import os
from rdkit import Chem
import shutil


def not_ligand_chain():
    del_list = ['HOH', 'ZN', 'MN', ' MG', 'MG ', ' AL', 'AL ', 'FE ', ' FE', ' LI', 'LI ', ' NA',
                'NA ', '  K', 'K  ', ' CA', 'CA ', ' SN', 'SN ', 'SO4', 'NO3', 'PO4', ' CL', 'CL ', 'NH3', 'NAG',
                'NDG', 'Y01', 'GOL', 'EDO', 'CPS', 'CLR', 'PLP', 'HH', 'HH ', 'MAL', 'MPO', 'ADP', 'BOG', 'CAC', 'HH2',
                ' DG', 'DG ', 'HEM', 'T  ', '  T', 'G  ', '  G', 'A  ', '  A', 'U  ', '  U', 'C  ', '  C', 'ATP', 'AMP',
                'DNP', 'ANP']
    return del_list


def wash_pdb(in_dir):
    error = 0
    changed = 0
    h_error = 0
    no_ligand = 0
    for pdb in os.listdir(in_dir):
        pdb_path = os.path.join(in_dir, pdb)
        with open(pdb_path) as f:
            txt = f.readlines()

        num = [idx for idx, i in enumerate(txt) if i.startswith('HETATM')]
        if len(num) == 0:
            os.remove(pdb_path)
            error += 1
            continue
        lines = [i for idx, i in enumerate(txt) if i.startswith('HETATM')]
        atoms = []
        for i in lines:
            atoms.append(i.split()[3])
        chains = list(set(atoms))
        count_chains = [atoms.count(i) for i in chains]
        best_chain = chains[count_chains.index(max(count_chains))]
        
        # 对多个配体链进行清洗
        if len(chains) > 1:
            txt2 = []
            for i in txt:
                if i.startswith('HETATM'):
                    if i.split()[3] == best_chain:
                        txt2.append(i)
                else:
                    txt2.append(i)
            with open(pdb_path, 'w') as f:
                f.writelines(txt2)
            changed += 1

        with open('refer.pdb', 'w') as f:
            f.writelines(lines)

        # 删除读不出refer的
        mol = Chem.MolFromPDBFile('refer.pdb')
        if mol is None:
            error += 1
            os.remove(pdb_path)
            continue

        with open('refer.pdb') as f:
            lines = f.readlines()
        if lines[0].split()[3] in not_ligand_chain():
            # print('发现敌人！')
            no_ligand += 1
            os.remove(pdb_path)
            continue
        '''
        obabel_cmd = 'obabel -ipdb refer.pdb -opdb -O refer2.pdb -h'
        os.system(obabel_cmd)
        mol = Chem.MolFromPDBFile('refer2.pdb')
        if mol is None:
            h_error += 1
            os.remove(pdb_path)
            continue

        # 删除配体不连续的
        check_list = list(range(num[0], num[-1]+1))
        if num != check_list:
            os.remove(pdb_path)
            error += 1

        # 删除读不出盒子的
        os.system('./lepro_linux_x86 %s' % pdb_path)
        with open('dock.in') as f:
            txt = f.read()
        if 'xmin' in txt:
            print(pdb)
            os.remove(pdb_path)
        '''
    print(in_dir, 'removed:', error)
    print(in_dir, 'h_error', h_error)
    print(in_dir, 'changed:', changed)
    print(in_dir, 'no_ligand:', no_ligand)


def rename_pdb(in_dir):
    for pdb in os.listdir(in_dir):
        ori_name = os.path.join(in_dir, pdb)
        new_name = os.path.join(in_dir, pdb.split('.')[0] + '.pdb')
        os.rename(ori_name, new_name)


def split_dir(in_dir):
    num = 1
    for idx, pdb in enumerate(os.listdir(in_dir)):
        ori_name = os.path.join(in_dir, pdb)
        shutil.copy(ori_name, 'sc'+str(num))
        if (idx+1) % 500 == 0:
            num += 1


if __name__ == '__main__':
    '''
    for i in range(1, 7):
        in_dir = 'sc' + str(i)
        wash_pdb(in_dir)
    '''
    in_dir = 'wash_multi_ligand_2274'
    # wash_pdb(in_dir)
    split_dir(in_dir)
    # rename_pdb(in_dir)

